<?php

require_once("Router/Router.php");
require_once("Router/Response.php");
require_once("Router/Route.php");
require_once("BancaDati/BancaDati.php");
require_once("User.php");

use Router\Response;
use Router\Router;
use Router\Route;
use BancaDati\BancaDati;

abstract class HTTP_STATUS_CODE
{
    const OK = 200;
    const CREATED = 201;
    const BAD_REQUEST = 400;
    const FORBIDDEN = 403;
    const NOT_FOUNT = 404;
}


$app = new Router("/DirektiveDesDons");
$db = new BancaDati();

$app->use("/", function (array &$req, Response $res) {
    if(isset($_COOKIE["TOKEN"])){
        $user = new User();
        if($user->loginWithToken($_COOKIE["TOKEN"])){
            $req["user"] = $user;
        }
    }
});

$app->get("/", function (array $req, Response $res) use ($db) {
    $res->send("Hello World");
    //var_dump($req["user"]);
});

/*
 * User
 */
$app->get("/user", function (array $req, Response $res) {
    if(isset($req["user"])){
        $res->json(["id" => $req["user"]->id, "username" => $req["user"]->username, "email" => $req["user"]->email]);
    }else {
        $res->json(["status" => HTTP_STATUS_CODE::FORBIDDEN, "message" => "You are not logged in. Goto '/login' to login"]);
    }
});
$app->get("/user/:id", function (array $req, Response $res) use ($db) {
    $db->select("utente", ["username" => $req["id"]]);
    $res->send("user " . $req["params"]["id"]);
});
$app->post("/signup", function (array $req, Response $res) use ($db) {
    $newUsername = $req["body"]["username"];
    $newPassword = $req["body"]["password"];
    $newEmail = $req["body"]["email"];
    $db->insert("utente", ["email" => "$newEmail", "parolaDordine" => "$newPassword", "nomeUtente" => "$newUsername"]);

    $res->send("Account Created", HTTP_STATUS_CODE::CREATED);
});

$app->post("/login", function( array $req, Response $res) use ($db) {
    $username = $req["body"]["username"];
    $password = $req["body"]["password"];

    $user = new User();
    $usertoken = $user->loginWithUsername($username, $password);

    if($usertoken){
        setcookie("TOKEN", $usertoken, time()+3600); // 1h
        $res->send("Login successful" . "token: " . $usertoken);
    }else{
        $res->send("Login failed", HTTP_STATUS_CODE::FORBIDDEN);
    }
});

/*
 * Ingredients
 */
$app->route("/ingredient")
    ->get(function (array $req, Response $res) use ($db) {
        $res->json(["data" => $db->select("ingredienti")]);
    })
    ->post(function (array $req, Response $res) use ($db) {
        $name = $req["body"]["name"];
        $calories = $req["body"]["calories"];
        $quantity = $req["body"]["quantity"];
        $unit = $req["body"]["unit"];
        $price = $req["body"]["price"];

        $unitInTable = $db->select("folla", ["unita" => $unit]);
        if(count($unitInTable) > 1){
            $res->json(["message" => "Unit: " . $unit . " does not exist. Please create unit first"], HTTP_STATUS_CODE::BAD_REQUEST);
            return;
        }
        $unitId = $unitInTable[0]["id"];

        $id = $db->insert("ingredienti", ["cognome" => "$name", "calorie" => "$calories", "quantita" => "$quantity", "follaID" => $unitId, "prezzo" => "$price"]);
        if(!$id){
            $res->json(["message" => "Something went wrong when creating the Ingredient"], HTTP_STATUS_CODE::BAD_REQUEST);
            return;
        }

        $res->json(["message" => "New ingredient has been listed",
            "ingredient" => $db->select("ingredienti", ["id" => $id])]);
    });
$app->get("/ingredient/:id", function (array $req, Response $res) use ($db) {
    $id = $req["params"]["id"];
    $ingredient = $db->select("ingredienti", ["id" => $id])[0];
    $res->json(["data" => $ingredient]);
});

/*
 * Unit
 */
$app->route("/unit")
    ->get(function (array $req, Response $res) use ($db) {
        $res->json(["data" => $db->select("folla")]);
    })
    ->post(function (array $req, Response $res) use ($db) {
        $name = $req["body"]["name"];

        if(!isset($name) || strlen($name) < 1 || strlen($name) > 200){
            $res->json(["message" => "Invalid Request. Please follow the the Documentation", HTTP_STATUS_CODE::BAD_REQUEST]);
            return;
        }

        $selectWithSameName = $db->select("folla", ["unita" => $name]);
        if(count($selectWithSameName) >= 1){
            $res->json(["message" => "Unit: " . $name ." already exists", "data" => $selectWithSameName[0]], HTTP_STATUS_CODE::BAD_REQUEST);
            return;
        }

        $newUnitId = $db->insert("folla", ["unita" => $name]);
        $res->json(["message" => "Unit: '" . $name . "' created", "data" => $db->select("folla", ["id" => $newUnitId])[0]], HTTP_STATUS_CODE::CREATED);
    });
$app->get("/unit/:id", function (array $req, Response $res) use ($db) {
    $id = $req["params"]["id"];
    $ingredient = $db->select("folla", ["id" => $id])[0];
    $res->json(["data" => $ingredient]);
});

$app->start();